#==========================================================
# Copyright @ 2014 Puppet Labs, LLC
# Redistribution prohibited.
# Address: 308 SW 2nd Ave., 5th Floor Portland, OR 97204
# Phone: (877) 575-9775
# Email: info@puppetlabs.com
#
# Please refer to the LICENSE.pdf file included
# with the Puppet Enterprise distribution
# for licensing information.
#==========================================================

#===[ Summary ]============================================
# This ruby script uses the Node Classifier to classify
# Puppet Enterprise.
#==========================================================

#===[ Functions ]==========================================

require 'json'
require 'net/https'
require 'uri'


class PEClassifier

  DEFAULT_NC_GROUP = '00000000-0000-4000-8000-000000000000'

  def initialize(env)
    @master = env['q_puppetagent_server'] # master
    @puppetdb = env['q_puppetdb_hostname'] # puppetdb
    @puppetdb_port = env['q_puppetdb_port'] # puppetdb_port
    @puppetdb_database_name= env['q_puppetdb_database_name']
    @puppetdb_database_user= env['q_puppetdb_database_user']
    @console = env['q_puppetagent_certname'] # console
    @console_port = env['q_puppet_enterpriseconsole_httpd_port'] # console_port
    @database_install = env['q_database_install'] == 'y'
    @database = env['q_database_host'] # database
    @database_port = env['q_database_port'] # database_port
    @platform_puppet_class = env['t_platform_puppet_class']
    @is_upgrade = env['IS_UPGRADE'] == 'y'
    @pe_version = "#{env['CURRENT_PE_MAJOR']}.#{env['CURRENT_PE_MINOR']}".to_f
    @tarball_server = env['q_tarball_server']
    @ca_cert_file = '/etc/puppetlabs/puppet/ssl/certs/ca.pem'
    @host_private_key_file = "/opt/puppet/share/puppet-dashboard/certs/#{env['q_puppetagent_certname']}.private_key.pem"
    @host_cert_file = "/opt/puppet/share/puppet-dashboard/certs/#{env['q_puppetagent_certname']}.cert.pem"
  end

  def build_auth(uri)
    https = Net::HTTP.new(uri.host, uri.port)
    https.use_ssl = true
    https.ca_file =  @ca_cert_file
    https.key = OpenSSL::PKey::RSA.new(File.read(@host_private_key_file))
    https.cert = OpenSSL::X509::Certificate.new(File.read(@host_cert_file))
    https.verify_mode = OpenSSL::SSL::VERIFY_PEER
    https
  end

  def configure_pe_repo
    @master_group['classes'].merge!('pe_repo' => {})
    @master_group['classes'].merge!("pe_repo::platform::#{@platform_puppet_class}" => {})

    # If they've set a tarball server (via answer file if this is split
    # install), use it. Otherwise leave it unset to use the default.
    if @tarball_server
      @master_group['classes']['pe_repo'].merge!('base_path' => @tarball_server)
    end
  end

  def add_license
    @console_group['classes'].merge!('puppet_enterprise::license' => {})
  end

  def add_broker
    @brokers_group['classes'].merge!("puppet_enterprise::profile::amq::broker" => {})
  end

  # High level function classifies both the console and master with the appropriate mcollective profiles
  # so that they can be set up correctly.
  def add_mcollective
    @master_group['classes'].merge!("puppet_enterprise::profile::master::mcollective" => {})
    @master_group['classes'].merge!("puppet_enterprise::profile::mcollective::peadmin" => {})
    @console_group['classes'].merge!("puppet_enterprise::profile::mcollective::console" => {})
  end

  def add_console_prune
    @console_group['classes'].merge!('pe_console_prune' => {'prune_upto' => '30'})
  end

  def add_ca
    @ca_group['classes'].merge!("puppet_enterprise::profile::certificate_authority" => {})
  end

  def add_master
    @master_group['classes'].merge!("puppet_enterprise::profile::master" => {})
  end

  def add_puppetdb
    profile_puppetdb = {}
    profile_puppetdb['database_properties'] = '' if ! @database_install
    @puppetdb_group['classes'].merge!("puppet_enterprise::profile::puppetdb" => {})
  end

  def add_console
    @console_group['classes'].merge!("puppet_enterprise::profile::console" => {})
  end

  def post_group(group)
    nc_api_url = "https://#{@console}:4433/classifier-api"
    uri = URI.parse("#{nc_api_url}/v1/groups")
    https = build_auth(uri)

    request = Net::HTTP::Post.new(uri.request_uri)
    request.body = group.to_json

    # This is required to send data to NC
    request["Content-Type"] = "application/json"

    response = https.request(request)
    if response.code.to_i >= 400
      puts "An error has occured: HTTP #{response.code}, #{response.body}"
      exit 0
    end
    # Return the uuid from redirect "/classifier-api/v1/groups/474a7cea-8b32-4990-b65b-1f841d030fcc"
    response['location'].split("/")[-1]
  end

  def merge_parent(group,uuid)
      group.merge!({'parent' => uuid })
  end

  def create_groups
    # make master group
    @master_group = {
      'name'    => "PE Master",
      'rule'    => ["or", ["=", "name", @master]], # pinned node
      'classes' => {},
    }
    # make puppetdb group
    @puppetdb_group = {
      'name'    => "PE PuppetDB",
      'rule'    => ["or", ["=", "name", @puppetdb]], # pinned node
      'classes' => {},
    }
    # make console group
    @console_group = {
      'name'    => "PE Console",
      'rule'    => ["or", ["=", "name", @console]], # pinned node
      'classes' => {},
    }

    @brokers_group = {
      'name'    => "PE ActiveMQ Broker",
      'rule'    => ["or", ["=", "name", @master]], # pinned node
      'classes' => {},
    }

    @ca_group = {
      'name'    => "PE Certificate Authority",
      'rule'    => ["or", ["=", "name", @master]], # pinned node
      'classes' => {},
    }
  end

  def create_infrastructure
    # make infrastructure group
    @infrastructure_group = {
      'name'    => "PE Infrastructure",
      'parent'  => DEFAULT_NC_GROUP,
      'classes' => {},
    }
    if ! @is_upgrade
      @infrastructure_group['classes'].merge!("puppet_enterprise" => {
        'certificate_authority_host'   => @master,
        'puppet_master_host'           => @master,
        'console_host'                 => @console,
        'console_port'                 => @console_port,
        'puppetdb_host'                => @puppetdb,
        'puppetdb_port'                => @puppetdb_port,
        'puppetdb_database_name'       => @puppetdb_database_name,
        'puppetdb_database_user'       => @puppetdb_database_user,
        'database_host'                => @database,
        'database_port'                => @database_port,
        'mcollective_middleware_hosts' => [@master],
        'database_ssl'                 => @database_install || @puppetdb == @database,
      })
    end
  end

  def create_mcollective
    # make mcollective group

    # This rule is specific because the is_pe fact is not correctly
    # being used by the classifier. This rule is basically the same
    # thing. If pe_version is empty, then it isn't PE.
    @mcollective_group = {
      'name'    => "PE MCollective",
      'rule'    => ["and",["~",["fact","pe_version"],".+"]], # pinned node
      'classes' => {"puppet_enterprise::profile::mcollective::agent" => {}},
    }
  end

  def save_groups
    # Create the PE Infrastructure group
    parent = post_group(@infrastructure_group)

    # Create the children of the PE Infrastructure group
    [
     @master_group,
     @puppetdb_group,
     @console_group,
     @mcollective_group,
     @brokers_group,
     @ca_group,
    ].each do |child|
      group = merge_parent(child,parent)
      post_group group
    end

  end

  # Classify PE
  def classify
    create_groups
    create_infrastructure
    create_mcollective

    if ! @is_upgrade
      add_broker
      add_mcollective
      add_ca
      add_master
      add_puppetdb
      add_console

      # Add the pe_console_prune class to the puppet_console group but only on new
      # installs as we don't want to surprise the user by pruning the DB without
      # notice within a day after an upgrade.
      add_console_prune

      # Classify the console with the puppet_enterprise::license class, to sync
      # the license file from the master. This is necessary for proper
      # behavior of the license app, so we do it unconditionally.
      add_license

    end

    # Classify the master with the pe_repo classes, only if this is a fresh
    # install or an upgrade from less than 3.2. We won't enforce it on
    # future upgrades because users may choose to unclassify the master
    # with these classes and we want to respect that.
    if ! @is_upgrade || @pe_version < 3.2
      # Setup the frictionless agent install repo on the master
      configure_pe_repo
    end

    save_groups
  end
end

if __FILE__ == $0
  PEClassifier.new(ENV).classify
  exit 0
end
